'''
    Created on Mar 7, 2015
    @author: jldupont
'''
import netifaces as ni

def get_eth0_ipv4():
    """
    Extracts the IPv4 address associated with eth0 interface
    """
    
    eth0 = ni.ifaddresses('eth0')[ni.AF_INET][0]
    return eth0['addr']



"""
    Created on 2012-01-20
    @author: jldupont
"""

from twisted.internet import epollreactor
epollreactor.install()

from protocol import SSDP

from twisted.internet import reactor
from twisted.internet.task import LoopingCall


def loop(proto):
    """
    The main loop
    """
    proto.sendNotify()
    


def run(usn=None, nt=None, loc=None, noloop=False, delay=30, eth0 = None):

    ##
    ## Substitute the eth0 IPv4 address
    ##
    loc = loc % {"eth0": eth0}

    proto = SSDP(usn=usn, nt=nt, location=loc, max_age=3600, disable_receive = True)

    ##
    ## Assigns a transport to the protocol instance
    ##    
    reactor.listenMulticast(SSDP.PORT, proto, listenMultiple=True)
    
    lc = LoopingCall(lambda:loop(proto))
    lc.start(delay, now=True)
    
    ##
    ## Because the LoopingCall is at least executed once,
    ##  we can safely exit here
    ##
    if not noloop:
        reactor.run()



"""
    Created on 2012-01-20
    @author: jldupont
"""

from twisted.internet import epollreactor
epollreactor.install()

from protocol import SSDP

from twisted.internet import reactor


def run(*p, **k):
    
    reactor.listenMulticast(SSDP.PORT 
                            ,SSDP()
                            ,listenMultiple=True)
    reactor.run()



"""
    Created on 2012-02-12
    @author: jldupont
"""
import logging, types

def dnorm(d):
    """
    Normalize dictionary
    
    >>> dnorm({"SoMeKeY":"  spaces  "})
    {'somekey': 'spaces'}
    """
    r={}
    for e in d:
        try:    r[e.lower()]=d[e].strip()
        except: r[e.lower()]=d[e]
    return r

def info_dump(d, align):
    fmt="%-"+str(align)+"s : %s"

    if type(d)==types.DictionaryType:        
        for key in d:
            logging.info(fmt % (key, d[key]))
            
    if type(d)==types.ListType:
        for el in d:
            key, value=el
            logging.info(fmt % (key, value))

def prepare_callable(module_name, function_name):
    
    ### just import this module if required to
    ### it is not present in python < 2.7
    import importlib
    
    try:
        mod=importlib.import_module(module_name)
    except:
        raise Exception("Can't import module '%s'" % module_name)
        
    try:
        fnc=getattr(mod, function_name)
    except:
        raise Exception("Module '%s' doesn't have a 'run' function" % mod)
    
    if not callable(fnc):
        raise Exception("Can't call 'run' function of callable '%S'" % module_name)
    
    return (mod, fnc)



'''
Created on Mar 7, 2015

@author: jldupont
'''
import json
import logging

from twisted.internet.protocol import DatagramProtocol


class SSDP(DatagramProtocol, object):
    """
    The SSDP protocol
    """
    ADDRESS = "239.255.255.250"
    PORT = 1900
    
    INTERESTING_TOKENS = ['location', 'server', 'nt', 'usn']
    
    NOTIFY_PACKET="""NOTIFY * HTTP/1.1\r
    Host: 239.255.255.250:1900\r
    Cache-Control: max-age=%(max_age)d\r
    Location: %(location)s\r
    Server: jldssdp UPnP/1.0\r
    NTS: ssdp:alive\r
    NT: %(nt)s\r
    USN: %(usn)s\r\n\r\n
    """
    
    def __init__(self, usn=None, nt=None, location=None, max_age=None, disable_receive=False):
        super(SSDP, self).__init__()
        
        self.usn = usn
        self.nt  = nt
        self.loc = location
        self.max_age = max_age
        
        self.disable_receive = disable_receive
    
    def startProtocol(self):
        """
        Called after protocol has started listening.
        """        
        # We don't want this to go very far
        self.transport.setTTL(1)
        
        # Join a specific multicast group:
        self.transport.joinGroup( self.ADDRESS )
        
    def sendNotify(self):
        """
        Sends a NOTIFY SSDP UDP packet
        """
        d = { "max_age": self.max_age, "nt": self.nt, "usn": self.usn, "location":self.loc }
        pkt = self.NOTIFY_PACKET % d
        self.transport.write(pkt, (self.ADDRESS, self.PORT))
        
            
    def datagramReceived(self, data, (host, port)):
        """
        Example:
        
        NOTIFY * HTTP/1.1
        Host: 239.255.255.250:1900
        Cache-Control: max-age=60
        Location: http://192.168.1.1:1780/InternetGatewayDevice.xml
        NTS: ssdp:alive
        Server: POSIX, UPnP/1.0 linux/5.10.56.51
        NT: urn:schemas-upnp-org:device:LANDevice:1
        USN: uuid:DF6DC556-C32F-8E72-45EB-B7CDDA382DB9::urn:schemas-upnp-org:device:LANDevice:1
        """
        
        if self.disable_receive:
            return
        
        try:

            lines = data.split("\n")
            lines = map(lambda line:line.strip(), lines)
            lines = filter(lambda line:len(line)>0, lines)
    
            if not self.is_notify(lines):
                return
            
            entry = self.keep_interesting_tokens(lines)
            
            print json.dumps( entry )

        except:
            logging.error("Data: %s" % repr(data))
        

    def is_notify(self, lines):
        
        first_line = lines[0]
        first_line_token = first_line.split(" ")
        first_token = first_line_token[0].lower()
        
        return first_token == "notify"


    def keep_interesting_tokens(self, lines):
        
        r = {}
        
        for line in lines:
            tokens = line.split(":", 1)
            start_token = tokens[0].lower()
            
            if start_token in self.INTERESTING_TOKENS:
                r[start_token] = tokens[1].strip()
                
        return r




#!/usr/bin/env python
"""
    Jean-Lou Dupont's SSDP scripts
    
    Created on 2015-02-28
    @author: jldupont
"""
__author__  ="Jean-Lou Dupont"
__version__ ="0.2"

DESC="""
Overview
--------

Collection of SSDP related scripts

* jldssdp-discover:  listen for SSDP NOTIFY messages
* jldssdp-notify:    send SSDP NOTIFY message, optionally in a loop with delay
"""


from distutils.core import setup
from setuptools import find_packages


setup(name=         'jldssdp',
      version=      __version__,
      description=  'Collection of SSDP related tools',
      author=       __author__,
      author_email= 'jl@jldupont.com',
      url=          'https://github.com/jldupont/jldssdp',
      package_dir=  {'': "src",},
      packages=     find_packages("src"),
      scripts=      ['src/scripts/jldssdp-discover'
                     ,'src/scripts/jldssdp-notify',
                     ],
      zip_safe=False
      ,long_description=DESC
      ,install_requires=[ "argparse", "twisted", "netifaces"
                         ]
      )

#############################################

f=open("latest", "w")
f.write(str(__version__)+"\n")
f.close()

